<?php

namespace Drupal\config_layers\EventSubscriber;

use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\config_layers\ConfigLayerManager;
use Drupal\config_layers\Entity\ConfigLayer;
use Drupal\config_layers\Config\FilteredStorage;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Config subscriber to allow automatic import of active configuration into layer.
 */
class ConfigSubscriber implements EventSubscriberInterface {

  const IMPORT_PRIORITY = 10;
  const EXPORT_PRIORITY = -10;

  /**
   * The Drupal Kernel.
   *
   * @var \Drupal\Core\ConfigLayerManager
   */
  protected $layerManager;

  /**
   * ConfigSubscriber constructor.
   *
   * @param \Drupal\config_layers\ConfigLayerManager $layerManager
   *   The layer manager.
   */
  public function __construct(ConfigLayerManager $layerManager) {
    $this->layerManager = $layerManager;
  }

  /**
   * Import new configuration into layers listening to events.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The event to process.
   * @param string $eventName
   *   Name of the event.
   */
  public function importConfig(ConfigCrudEvent $event, $eventName) {
    $saved_config = $event->getConfig();
    $config_name = $saved_config->getName();
    $keys = [$config_name];

    $layer_ids = $this->layerManager->getLayers();
    $layers = ConfigLayer::loadMultiple($layer_ids);

    foreach ($layers as $layer) {
      if (in_array($eventName, $layer->getImportEvents())) {
        switch ($eventName) {
          case ConfigEvents::SAVE:
            // Optimization: import only the saved key.
            $source = new FilteredStorage($layer->getSourceStorage(), $keys);
            $layer->import($source, ConfigLayer::UPDATE_MODE_COPY);
            break;

          case ConfigEvents::DELETE:
            // Optimization: delete only the key.
            $layer->getDatabaseStorage()->delete($config_name);
            break;

          default:
            $keys = NULL;
            $layer->import();
        }
      }
    }

    $this->layerManager->mergeLayers($layers, $keys);
  }

  /**
   * Export configuration to layers listening to events.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The event to process.
   * @param string $eventName
   *   Name of the event.
   */
  public function exportConfig(ConfigCrudEvent $event, $eventName) {
    $layer_ids = $this->layerManager->getLayers();
    $layers = ConfigLayer::loadMultiple($layer_ids);
    foreach ($layers as $layer) {
      if (in_array($eventName, $layer->getExportEvents())) {
        $layer->export();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {

    $events = [];
    if (\Drupal::hasContainer() && \Drupal::entityTypeManager()->hasHandler('config_layer', 'storage')) {
      $layers = \Drupal::entityTypeManager()->getStorage('config_layer')->loadMultiple();
      $import_events = [];
      $export_events = [];
      foreach ($layers as $layer) {
        if ($layer->status()) {
          $import_events += $layer->getImportEvents();
          $export_events += $layer->getExportEvents();
        }
      }

      foreach (array_unique($import_events) as $event) {
        $events[$event][] = ['importConfig', static::IMPORT_PRIORITY];
      }
      foreach (array_unique($export_events) as $event) {
        $events[$event][] = ['exportConfig', static::EXPORT_PRIORITY];
      }
    }
    return $events;
  }

}
